import urllib
import urlparse

def add_get_parameters(url, **kwargs):
    url_parts = list(urlparse.urlparse(url))
    query = dict(urlparse.parse_qsl(url_parts[4], keep_blank_values=True, strict_parsing=False))
    query.update(**kwargs)
    url_parts[4] = urllib.urlencode(query)
    return urlparse.urlunparse(url_parts)



__author__ = 'msakharova'

def get_matched_string(match):
    """
    :return: string portion which matched specified match (not the initial search string which is represented by match.string)
    """
    return match.string[match.start():match.end()]
def remove_group_from_match(match, group):
    """
    :return: string portion which matched specified match without text portion matched by the specified group
    """
    return "%s%s" % (match.string[match.start():match.start(group)], match.string[match.end(group):match.end()])

def replace_groups_in_match(match, group_values):
    """
    :param group_values: a list-like object of tuples (group_name, str_value_with_which_group_has_to_be_replaced)
    :return: string portion which matched specified match, but with text portions which matched group_names from parameter group_values replaced by values_with_which_groups_have_to_be_replaced from group_values.
    """
    result = get_matched_string(match)
    group_names_to_replace = [group_name for group_name, value in group_values]
    # spans identifying coordinates of groups from group_values in matched string (they need to be shifted to start of the match)
    spans_to_replace = [(match.start(group_name) - match.start(), match.end(group_name) - match.start()) for group_name in group_names_to_replace]
    spans_to_replace_w_values = [(spans_to_replace[i], group_values[i]) for i in range(len(group_values))]
    sorted_spans_with_group_values = sorted(spans_to_replace_w_values, key=lambda span_and_value: span_and_value[0], reverse=True)
    for span, (group, value) in sorted_spans_with_group_values:
        prefix = result[:span[0]]
        postfix = result[span[1]:]
        result = "%s%s%s" % (prefix, value, postfix)
    return result



#!/usr/bin/env python
"""
Installation script:

To release a new version to PyPi:
- Ensure the version is correctly set in django_blakey_utils.__init__.py
- Run: python setup.py sdist upload
"""

from setuptools import setup, find_packages

from django_blakey_utils import get_version

setup(name='django-blakey-utils',
    version=get_version().replace(' ', '-'),
    url='https://github.com/tangentlabs/django-blakey-utils',
    author="Tangent Labs",
    author_email="mariia.sakharova@tangentlabs.co.uk",
    description="Common utilities for Blakey project",
    long_description=open('README.rst').read(),
    keywords="Utilities, Blakey",
    license='BSD',
    platforms=['linux'],
    packages=find_packages(exclude=["*.tests"]),
    install_requires=[
        'Django==1.4'
    ],
    # See http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=['Development Status :: 1 - Planning',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Operating System :: Unix',
                 'Programming Language :: Python']
)



from django.utils.unittest.util import safe_repr

def assert_dictionary_contains_values(test_case, dictionary_name, dictionary, **kwargs):
    """
    Asserts that specified dictionary contains values specified in keyword arguments
    :param test_case: TestCase-like object which has assertTrue and assertEqual methods
    :param dictionary_name: name to output in case of error.
    :type dictionary_name: str
    :param dictionary: dictionary to validate
    :type dictionary: dict
    :param **kwargs: keyword arguments against which dictionary should be validated.
    """
    for k, v in kwargs.items():
        test_case.assertTrue(dictionary.has_key(k), "%s is not present in dictionary %s" % (k, dictionary_name))
        test_case.assertEqual(dictionary[k], v, "key %s; dictionary %s" % (k, dictionary_name))

def assert_dictionary_contains_values_from(test_case, dictionary_name, dictionary, from_dictionary):
    """
    Asserts that specified dictionary contains all (key, value) pairs from specified from_dictionary
    :param test_case: TestCase-like object which has assertTrue and assertEqual methods
    :param dictionary_name: name to output in case of error.
    :type dictionary_name: str
    :param dictionary: dictionary to validate
    :type dictionary: dict
    :param from_dictionary: dictionary with values which must be in specified param dictionary
    :type from_dictionary: dict
    """
    assert_dictionary_contains_values(test_case, dictionary_name, dictionary, **from_dictionary)

def assert_equal_line_by_line(test_case, expected_text, text, msg = None):
    """
    Asserts that text equals expected_text. Compares expected_text to text line-by-line for better output. Convenient for long texts.
    :param test_case: TestCase-like object which has assertTrue and assertEqual methods
    :param expected_text: expected value
    :type expected_text: str
    :param text: actual value
    :type text: str
    :param msg: optional message to append to the output of error
    :type msg: str
    """
    msg = msg and "\n%s" % msg or ""
    expected_lines = expected_text.splitlines(True)
    text_lines = text.splitlines(True)
    expected_lines_count = len(expected_lines)
    lines_error = ""
    if expected_lines_count != len(text_lines):
        lines_error = "Expected number of lines: %s, but was: %s" % (expected_lines_count, len(text_lines))
    for i in range(min(expected_lines_count, len(text_lines))):
        actual_line = text_lines[i]
        expected_line = expected_lines[i]
        test_case.assertEqual(expected_line, actual_line, "\nLine %s.\nExpected: %s\nActual:   %s\n%s%s" % (i, safe_repr(expected_line), safe_repr(actual_line), lines_error, msg))




import re

class MarkdownPatterns:
    #![Alt text](/path/to/img.jpg "Optional title")
    IMAGE_INLINE_PATTERN = re.compile(r"""!\[(?P<name>([^\]]|\n)*?)\]   # ![Alt text]
            (?P<url_data>
                \(                                                  # (
                    (?P<url>[^\s\'\"]*)                             # /path/to/img.jpg
                    (?P<title_data>
                        \s*?                                         # whitespaces
                        ([\"\'](?P<title>(.|\n)*)[\"\'])?                # "Optional title", optional
                    )
                \)                                                  # )
            )""",  re.VERBOSE)
    #[an example](http://example.com/ "Title") for links that are like:
    LINK_INLINE_PATTERN = re.compile(r"""([^!]|^)\[(?P<name>([^\]]|\n)*?)\] # [an example]
            (?P<url_data>
                \(                                              # (
                    (?P<url>[^\s\'\"]*)                         # http://example.com/
                    (?P<title_data>
                        \s*?                                     # whitespaces
                        ([\"\'](?P<title>(.|\n)*)[\"\'])?            # "Title", optional
                    )
                \)                                              # )
            )""",  re.VERBOSE)
    GROUP_URL, GROUP_NAME, GROUP_TITLE, GROUP_URL_DATA, GROUP_TITLE_DATA = ("url", "name", "title", "url_data", "title_data")
